/*
 * lcd.c
 *
 * Created: 03.11.2011 17:50:30
 *  Author: gobe
 */ 

#include "lcd.h"
#include "config.h"
#include <avr/io.h>
#include <util/delay.h>
#include <string.h>
#include <stdlib.h>

void SPI_MasterInit(void)
{
    /* Set MOSI and SCK output, all others input */
    LCD_DDR = (1<<LCD_SDI)|(1<<LCD_CLK)|(1<<LCD_CSB)|(1<<LCD_RS);
    /* Enable SPI, Master, set clock rate fck/16 */
    //SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);
	
	SPCR |= (0 << SPIE) | /* SPI Interrupt disable */
           (1 << SPE)  | /* SPI Enable */
           (0 << DORD) | /* Data Order: MSB first */
           (1 << MSTR) | /* Master mode */
           (0 << CPOL) | /* Clock Polarity: CTRL_SCK low when idle */
           (0 << CPHA) | /* Clock Phase: sample on rising CTRL_SCK edge */
           (0 << SPR1) | /* Clock Frequency: f_OSC / 4  => 4Mbit */
           (0 << SPR0);
    SPSR &= ~(1 << SPI2X); /* No doubled clock frequency */
	_delay_us(20);
}

void SPI_MasterTransmit(char cData)
{
    /* Start transmission */
    SPDR = cData;
    /* Wait for transmission complete */
    while (!(SPSR & (1<<SPIF)))
        ;
}

void LCDInit(void)					// DOGM162
{
	_delay_ms(100);					// LCD traege
	LCD_PORT &= ~(1 << LCD_CSB);	// low
    _delay_ms(100);					// LCD traege
    SPI_MasterInit();
    _delay_ms(10);
	
	LCD_PORT &= ~(1 << LCD_RS);		// low
	
    SPI_MasterTransmit(0x39);
    _delay_ms(10);
	LCDToggle_Control();
	
    SPI_MasterTransmit(0x1C);
    _delay_ms(10);
	LCDToggle_Control();
	
    SPI_MasterTransmit(0x52);
    _delay_ms(10);
	LCDToggle_Control();
	
    SPI_MasterTransmit(0x69);
    _delay_ms(10);
	LCDToggle_Control();
	
    SPI_MasterTransmit(0x74);
    _delay_ms(10);
	LCDToggle_Control();
	
    SPI_MasterTransmit(0x38);
    _delay_ms(10);
	LCDToggle_Control();
	
    SPI_MasterTransmit(0b00001100);		// Display on, Cursor Settings
    _delay_ms(10);
	LCDToggle_Control();
	
    SPI_MasterTransmit(0x01);
    _delay_ms(10);
	LCDToggle_Control();
	
	SPI_MasterTransmit(0x06);
    _delay_ms(10);
	LCDToggle_Control();
	
}

void LCDToggle_Control(void)
{
	LCD_PORT |= (1 << LCD_RS);			// high
	LCD_PORT &= ~(1 << LCD_RS);			// low
}

void LCDToggle_Write(void)
{
	LCD_PORT &= ~(1 << LCD_RS);			// low
	LCD_PORT |= (1 << LCD_RS);			// high
}

void LCDWriteString(char* mystring)
{
    int i;

    for (i=0;i<strlen(mystring);i++)
    {
        LCD_PORT |= (1 << LCD_RS);		// high
        SPI_MasterTransmit(mystring[i]);
        _delay_ms(2);
		LCDToggle_Write();
    }
}

void LCDWriteNumber(unsigned char num)
{
    int i;
	char buffer [4];		//max 3 chars
	
	itoa(num, buffer, 10);
	
    for (i=0;i<strlen(buffer);i++)
    {
        LCD_PORT |= (1 << LCD_RS);		// high
        SPI_MasterTransmit(buffer[i]);
        _delay_ms(2);
		LCDToggle_Write();
    }
}

void LCDWriteChar(char c)
{
	LCD_PORT |= (1 << LCD_RS);			// high
	SPI_MasterTransmit(c);
    _delay_ms(2);
	LCDToggle_Write();
}

void LCDSetCursor (unsigned char pos)
{
	LCD_PORT &= ~(1 << LCD_RS);			// Low								
	SPI_MasterTransmit(128+pos);		// Zeile 1: 1-39 - Zeile 2: 40+								
	_delay_ms(20);						
	LCDToggle_Control();
}

void LCDClr(void)
{
	LCD_PORT &= ~(1 << LCD_RS);			// Low								
	SPI_MasterTransmit(0x01);							
	_delay_ms(20);						
	LCDToggle_Control();
}
